#include <windows.h>
#include <stdio.h>
#include <malloc.h>
#include "BladeDll.h"

BOOL loadVorbEnc( void );
BOOL openFiles( char *lpszInFile, char *lpszOutFile );

static HANDLE hBladeDll = NULL;
static BEINITSTREAM   beInitStream;
static BEENCODECHUNK  beEncodeChunk;
static BEDEINITSTREAM beDeinitStream;
static BECLOSESTREAM  beCloseStream;
static BEVERSION      beVersion;
static FILE *fpIn;
static FILE *fpOut;

int main( int argc, char **argv )
{
  if ( argc != 3 )
    {
      printf( "Usage: test <infile.wav> <outfile.ogg>\n" );
      return 0;
    }

  if ( !loadVorbEnc() )
    {
      printf( "Error loading vorb_enc.dll\n" );
      return 0;
    }


  if ( !openFiles( argv[1], argv[2] ) )
    {
      printf( "Unable to open input and/or output files\n" );
      return 0;
    }

  encodeThePuppy();

  fclose( fpIn );
  fclose( fpOut );

  return 0;
}


BOOL loadVorbEnc( void )
{
#if 1
  hBladeDll = LoadLibrary( "vorb_enc.dll" );
#else
  hBladeDll = LoadLibrary( "BladeEnc.dll" );
#endif
  if ( !hBladeDll )
    return FALSE;

  beInitStream = (BEINITSTREAM)GetProcAddress( hBladeDll, "beInitStream" );
  beEncodeChunk = (BEENCODECHUNK)GetProcAddress( hBladeDll, "beEncodeChunk" );
  beDeinitStream = (BEDEINITSTREAM)GetProcAddress( hBladeDll, "beDeinitStream" );
  beCloseStream = (BECLOSESTREAM)GetProcAddress( hBladeDll, "beCloseStream" );
  beVersion = (BEVERSION)GetProcAddress( hBladeDll, "beVersion" );

  if ( !beInitStream || !beEncodeChunk || !beDeinitStream ||
       !beCloseStream || !beVersion )
    {
      FreeLibrary( hBladeDll );
      hBladeDll = FALSE;
      return FALSE;
    }
  return TRUE;
}



BOOL openFiles( char *lpszInFile, char *lpszOutFile )
{
  fpIn = fopen( lpszInFile, "rb" );
  if ( !fpIn )
    return FALSE;

  fpOut = fopen( lpszOutFile, "wb" );
  if ( !fpOut )
    return FALSE;

  return TRUE;
}



int encodeThePuppy( void )
{
  HBE_STREAM hbeStream;
  BE_CONFIG b;
  BE_ERR err;
  DWORD dwSamples, dwMP3BufferSize, dwSampleBytes;
  BYTE *pInput, *pOutput;
  DWORD dwNumRead;

  ZeroMemory( &b, sizeof(b) );
  b.dwConfig = BE_CONFIG_MP3;
  b.format.mp3.dwSampleRate = 44100;
  b.format.mp3.byMode = BE_MP3_MODE_STEREO;
  b.format.mp3.wBitrate = 128;
  b.format.mp3.bCRC = FALSE;

  err = beInitStream( &b, &dwSamples, &dwMP3BufferSize, &hbeStream );
  dwSampleBytes = 2 * dwSamples;

  pInput = (unsigned char *)malloc( dwSampleBytes );
  pOutput = (unsigned char *)malloc( dwMP3BufferSize );

  // skip the wav header
  fread( pInput, 1, 44, fpIn );
  while( !feof( fpIn ) )
    {
      DWORD dwNumRead;
      dwNumRead = fread( pInput, 1, dwSampleBytes, fpIn );

      if ( dwNumRead )
	{
	  if ( beEncodeChunk( hbeStream, dwNumRead/2, (PSHORT)pInput, pOutput, &dwNumRead ) != BE_ERR_SUCCESSFUL )
	    {
	      printf( "Error from Encoder... aborting!\n" );
	      goto abortPoint;
	    }
	  if ( dwNumRead )
	    {
	      fwrite( pOutput, 1, dwNumRead, fpOut );
	      printf( "wrote %d bytes\n", dwNumRead );
	    }
	}
    }

abortPoint:  
  beDeinitStream( hbeStream, pOutput, &dwNumRead );
  if ( dwNumRead )
    {
      fwrite( pOutput, 1, dwNumRead, fpOut );
      printf( "wrote %d bytes\n", dwNumRead );
    }
  beCloseStream( hbeStream );

  free( pInput );
  free( pOutput );

  return 0;
}
